#include <stdio.h>
#include <stdlib.h>
#include <complex.h>
#include <string.h>
#include <math.h>

#include "mkfilter.h"
#include "filter.h"

complex double filter_lowpass (struct filter_lowpass *filter, int order, double alpha, complex double input) {

  if (filter->order != order || filter->alpha != alpha) {
    if (order > FILTER_MAX_ORDER) {
      fprintf(stderr, "filter order too high\n");
      exit(1);
    };
    if (!(order & 1)) {
      fprintf(stderr, "write code for even order numbers\n");
      exit(1);
    };
    mkfilter(order, alpha, filter->xc, filter->yc);
    fprintf(stderr, "filter coefficients (order %d, alpha %f)\n", order, alpha);
    for (int i = 0; i < order + 1; i++) {
      fprintf(stderr, "xc[%d] = %0.24f\n", i, filter->xc[i]);
    };
    for (int i = 0; i < order + 1; i++) {
      fprintf(stderr, "yc[%d] = %0.24f\n", i, filter->yc[i]);
    };
    memset(filter->xv, 0, (FILTER_MAX_ORDER+1) * sizeof(complex double));
    memset(filter->yv, 0, (FILTER_MAX_ORDER+1) * sizeof(complex double));
    filter->order = order;
    filter->alpha = alpha;
  };

  memmove(filter->xv, filter->xv + 1, order * sizeof(complex double));
  memmove(filter->yv, filter->yv + 1, order * sizeof(complex double));

  filter->xv[order] = input;

  complex double sum = 0.0;
  for (int i = 0; i < order / 2; i++) {
    sum += filter->xc[i] * (filter->xv[i] + filter->xv[order-i]);
  };
  for (int i = 0; i < order; i++) {
    sum += filter->yc[i] * filter->yv[i];
  };

  double x = creal(sum);
  double y = cimag(sum);
  if (isnan(x)) x = 0.000123;
  if (isnan(y)) y = 0.000123;
  if (x > +1000.0) x = +1000.0;
  if (x < -1000.0) x = -1000.0;
  if (y > +1000.0) y = +1000.0;
  if (y < -1000.0) y = -1000.0;
  sum = x + y * I;

  filter->yv[order] = sum;

  return sum;

};
